@include portal-type-font-awesome-icon(ftw-events-eventfolder, calendar);
@include portal-type-font-awesome-icon(ftw-events-eventpage, calendar-o);
@include portal-type-font-awesome-icon(ftw-events-eventlistingblock, calendar-o);

.event-listingblock-moreitemslink {
  @extend .fa-icon;
  @extend .fa-caret-right;
  display: block;
}

.event-details {
  .when {
    margin-bottom: $margin-vertical;
    ul {
      @include list;
    }
  }
}

.event-row {
  @include list();
  margin-bottom: $margin-vertical;

  > li:last-child .event-item {
    margin-bottom: 0;
  }

  .event-item {
    @include no-link();
    @include clearfix();
    padding: 0;
    margin-bottom: $margin-vertical;

    &:hover {
      background-color: $color-gray-light;
    }

    .body {
      .event-more {
        display: inline;
        &:before {
          min-width: 0;
        }
      }
    }

    .byline {
      display: block;
      font-size: $font-size-small;
      color: $color-text-light;
    }

    .title {
      font-size: 1rem;
      font-weight: bold;
      display: block;
    }
  }
}

.ics-export {
    @extend .fa-icon;
    @extend .fa-calendar;
    margin-bottom: $margin-vertical;
    display: block;
    float: right;
}

// Hide empty event listing block for anonymous users.
.portlet.SimplelayoutPortlet .ftw-events-eventlistingblock.hidden {
  display: none;
}

// Add special border to empty listing block for authenticated users.
body.userrole-authenticated .portlet.SimplelayoutPortlet .ftw-events-eventlistingblock.hidden {
  @include boxshadow(0 0 20px 0 $color-warning);
  display: block;
}

// Add RSS icon to rss link in eventlisting block
.events-rss {
  @extend .fa-icon;
  @extend .fa-rss;
}
