.event-archive-portlet {
  ul {
    @include list-tree($indentation: $line-height-base);
  }

  .count {
    @include label-round();
  }

  .event-year {
    @extend .fa-icon;
    @extend .fa-angle-right;

    &.expanded {
      @extend .fa-icon;
      @extend .fa-angle-down;

      ~ .event-months {
        display: block;
      }
    }
  }

  .event-months {
    display: none;
  }
}
